// ------------------------------------------------------------------------ //
//                                                                          //
// File:    Key2Env.cpp                                                     //
// Author:  Dirk Schwarzmann                                                //
// Contact: Dirk.Schwarzmann@gmx.de                                         //
//                                                                          //
// Version: 1.1                                                             //
// Date:    15th Febuary 2001                                               //
//                                                                          //
// Purpose:                                                                 //
//          Reads a keyboard input and writes it to a batch file containing //
//          a command for assigning the input to an environment variable.   //
//                                                                          //
//          call: key2env [prompt] parameter batchfile                      //
//                                                                          //
// Bugs:                                                                    //
//                                                                          //
//       You have to press the ENTER key twice to accept the input.         //
//       This is due to the getline(...) command I used. Maybe you can      //
//       point me to a better solution?                                     //
//                                                                          //
// ------------------------------------------------------------------------ //

#include <iostream>
#include <fstream>
#include <string>
using namespace std;

void main(int argc, char* argv[]) {

  string envVar;
  string batchFile;

  if (argc <= 2 || argc > 4) {

    // Gib einen Hilfetext aus:
    cout << "Leitet die Tastatureingabe in eine Batchdatei, die die Umgebungsvariable setzt.\n";
    cout << "Aufruf: Key2Env [Aufforderungstext] Variablenname Batchdatei\n";
    cout << "Der A-Text muss statt Leerzeichen Unterstriche enthalten (werden ersetzt).\n";

  } else { // if (argc <= 2 || argc > 4)

    // Pruefe die Parameter:
    if (argc == 4) {

      // Es wurde ein Eingabeaufforderungstext (=Prompt) angegeben.
      // Dann ist der zweite Parameter ist die Umgebungsvariable:
      envVar = argv[2];

      // Und der dritte ist der Name der Batchdatei:
      batchFile = argv[3];

      // Gib das Prompt aus. Dazu werden vorher alle Unterstriche in Leerzeichen gewandelt:
      string prompt = argv[1];
      int pos = prompt.find("_");

      while (pos != string::npos) {
        prompt.replace(pos, 1, " ");
        pos = prompt.find("_");
      }

      cout << prompt;

    } else { // if (argc == 4)

      // Kein Prompt angegeben.
      // Damit ist der erste Parameter also schon die Umgebungsvariable:
      envVar = argv[1];

      // Und der zweite ist der Name der Batchdatei:
      batchFile = argv[2];
    }

    // Warte nun auf die Usereingabe und belege die Variable mit ihr:
    string userInput;

    // Der Delimiter \n erzwingt doppeltes Druecken von ENTER: :-(((
    getline(cin, userInput, '\n');

    // Baut die Ausgabezeile zusammen:
    envVar = "set " + envVar + "=" + userInput;

    ofstream outFile;
    
    outFile.open(batchFile.c_str(), ios::out);

    outFile.write(envVar.c_str(), envVar.length());

    outFile.close();

  } // else if (argc <= 2 || argc > 4)

} // void main(int, char*)