PROGRAM MSDOSPatchKeyboard;
(*
   Version 1.0 vom 18.08.1999
   Autor: Dirk Schwarzmann
   EMail: Dirk.Schwarzmann@gmx.de
   WWW:   http://www.informatik.uni-oldenburg.de/~rob/


   Beschreibung:
     Das Programm patcht die MS-DOS-Treiber KEYBOARD.SYS und KEYBRD2.SYS
     so, dass auf dem Nummernblock statt des Kommas ein Punkt erscheint.
     Ausserdem kann statt des normalen ShiftLock ein echtes CapsLock
     eingestellt werden.


   Parameter beim Aufruf (Reihenfolge beachten!):
     - Pfad (mit Laufwerksangabe) und Name des MSDOS-Treibers, meist
       KEYBOARD.SYS oder KEYBRD2.SYS.
     - "-Punkt" oder "-Komma" und / oder
       "-ShiftLock" oder "-CapsLock"
     - "-debug" (Fuer eine Bildschirmausgabe, wo das Byte gefunden wurde)

   Rueckgabewerte (Abfrage per "ERRORLEVEL" in einer Batchdatei):
     0 - Fehlerfreie Ausfuehrung
     1 - Falsche Parameteranzahl
     2 - Patchen war erfolglos

   ACHTUNG! Die Errorlevel muessen rueckwaerts, d.h. mit der hoechsten Zahl
   beginnend abgefragt werden, sonst wird immer "ok" erkannt!
   (Dies ist ein Problem von DOS und hat nichts mit diesem Programm zu tun)

*)


{$M 56080, 0, 327680} (* 50kB Stackgroesse *)

USES
  DOS, CRT;

CONST
  FileBufferSize = 47888; (* Groesse des internen Puffers, in den die Files
                             eingelesen werden *)

VAR
  DriverFile : STRING;
  FSize      : LONGINT;
  FileBuffer : ARRAY[1..FileBufferSize] OF BYTE;
  MustWrite  : BOOLEAN; (* Flag, ob die Datei geaendert wurde und daher ein
                           Zurueckschreiben ueberhaupt noetig ist. *)
  KommaPunkt : BYTE;    (* 0= Nichts patchen, 44= Komma, 46= Punkt *)
  ShiftCaps  : BYTE;    (* 0= Nichts patchen, 1= ShiftLock, 2= CapsLock *)
  debug      : BOOLEAN; (* Flag fuer den Debug-Modus *)

    (* ------------------------ Procedures ------------------------- *)

PROCEDURE ReadFileInBuffer(FileName:STRING);
(*
   Liest das Originalfile in den Buffer, damit es im Speicher bearbeitet
   werden kann.
*)

  VAR
    Source : FILE;
    RecRead : WORD;

  BEGIN (* proc.ReadFileInBuffer *)
    Assign(Source,FileName);
    Reset(Source, 1); (* Recordgroesse ist 1 *)
    FSize := FileSize(Source);
    REPEAT
      BlockRead(Source,FileBuffer,SizeOf(FileBuffer),RecRead);
    UNTIL (RecRead = 0);
    Close(Source);
  END; (* proc.ReadFileInBuffer *)


PROCEDURE WriteFileFromBuffer(FileName:STRING);
(*
   Schreibt den Buffer in die physikalische Datei zurueck.
*)

  VAR
    Source : FILE;

  BEGIN (* proc.WriteFileFromBuffer *)
    Assign(Source,DriverFile);
    Rewrite(Source, 1); (* Recordgre ist 1 *)
    BlockWrite(Source,FileBuffer,FSize);
    Close(Source);
  END; (* proc.WriteFileFromBuffer *)


PROCEDURE PatchKommaPunkt(Zeichen : BYTE);
  (* Punkt oder Komma in den Treiber einbauen.
     In "Zeichen" wird der Dezimalwert des Zeichens (44= Komma, 46= Punkt)
     uebergeben.
  *)

  VAR
    i : WORD;
    Found : BOOLEAN;

  BEGIN (* proc.PatchKommaPunkt *)
    (* Suche nach der richtigen Zeichenfolge: *)
    i := 1;
    Found := FALSE;
    REPEAT
      Inc(i);
      IF (FileBuffer[i]=0) AND (FileBuffer[i+1]=192) AND (FileBuffer[i+2]=1)
      AND (FileBuffer[i+3]=83) AND (FileBuffer[i+5]=0)
      AND (FileBuffer[i+6]=0) AND (FileBuffer[i+7]=49) AND (FileBuffer[i+8]=0)
      AND (FileBuffer[i+9]=5) AND (FileBuffer[i+10]=0) AND (FileBuffer[i+11]=20)
      AND (FileBuffer[i+12]=255) AND (FileBuffer[i+13]=255) THEN BEGIN
        (* Gefunden: Setze das neue Zeichen hier: *)
        IF debug THEN
          WriteLn('Komma/Punkt gefunden an Position (Dezimal): ',i);
        FileBuffer[i+4] := Zeichen;
        Found := TRUE;
      END;
    UNTIL Found OR (i = FSize-13);
    IF Found THEN
      MustWrite := TRUE;
  END; (* proc.PatchKommaPunkt *)


PROCEDURE PatchCapsLock;
  (* Das fehlerfreie CapsLock in den Treiber einbauen. *)

  VAR
    i : WORD;
    Found : BOOLEAN;

  BEGIN (* proc.PatchCapsLock *)
    (* Suche nach der richtigen Zeichenfolge: *)
    i := 551; (* vorher brauchen wir gar nicht zu suchen *)
    Found := FALSE;
    REPEAT
      Inc(i);
      IF (FileBuffer[i]=0) AND (FileBuffer[i+1]=192) AND (FileBuffer[i+2]=1)
      AND (FileBuffer[i+3]=83) AND (FileBuffer[i+5]=0)
      AND (FileBuffer[i+6]=0) AND (FileBuffer[i+7]=49) AND (FileBuffer[i+8]=0)
      AND (FileBuffer[i+9]=5) AND (FileBuffer[i+10]=0) AND (FileBuffer[i+11]=20)
      AND (FileBuffer[i+12]=255) AND (FileBuffer[i+13]=255)
      AND (FileBuffer[i-551] = 128) THEN BEGIN
        FileBuffer[i-551] := 0;
        FileBuffer[i-536] := 2;
        FileBuffer[i-531] := 1;
        FileBuffer[i-278] := 6;
        FileBuffer[i-243] := 5;
        IF debug THEN
          WriteLn('Caps gefunden an Position (Dezimal): ',i);
        Found := TRUE;
      END;
    UNTIL Found OR (i = FSize-13);
    IF Found THEN
      MustWrite := TRUE;
  END; (* proc.PatchCapsLock *)


PROCEDURE PatchShiftLock;
  (* Das normale ShiftLock in den Treiber einbauen. *)

  VAR
    i : WORD;
    Found : BOOLEAN;

  BEGIN (* proc.PatchShiftLock *)
    (* Suche nach der richtigen Zeichenfolge: *)
    i := 551; (* vorher brauchen wir gar nicht zu suchen *)
    Found := FALSE;
    REPEAT
      Inc(i);
      IF (FileBuffer[i]=0) AND (FileBuffer[i+1]=192) AND (FileBuffer[i+2]=1)
      AND (FileBuffer[i+3]=83) AND (FileBuffer[i+5]=0)
      AND (FileBuffer[i+6]=0) AND (FileBuffer[i+7]=49) AND (FileBuffer[i+8]=0)
      AND (FileBuffer[i+9]=5) AND (FileBuffer[i+10]=0) AND (FileBuffer[i+11]=20)
      AND (FileBuffer[i+12]=255) AND (FileBuffer[i+13]=255)
      AND (FileBuffer[i-551] = 0) THEN BEGIN
        FileBuffer[i-551] := 128;
        FileBuffer[i-536] := 1;
        FileBuffer[i-531] := 2;
        FileBuffer[i-278] := 5;
        FileBuffer[i-243] := 6;
        IF debug THEN
          WriteLn('Shift gefunden an Position (Dezimal): ',i);
        Found := TRUE;
      END;
    UNTIL Found OR (i = FSize-13);
    IF Found THEN
      MustWrite := TRUE;
  END; (* proc.PatchShiftLock *)


FUNCTION GermanUpcase(WorkStr:STRING):STRING;
(*
   Konvertiert in einem String alle Buchstaben samt Umlauten in
   Grossbuchstaben.
 *)

  VAR
    i : BYTE;

  BEGIN (* func.GermanUpcase *)
    FOR i := 1 To Length(WorkStr) DO BEGIN
      WorkStr[i] := Upcase(WorkStr[i]);
      CASE WorkStr[i] OF
        '' : WorkStr[i] := '';
        '' : WorkStr[i] := '';
        '' : WorkStr[i] := '';
      END; (* case WorkStr[i] *)
    END; (* for i *)
    GermanUpcase := WorkStr;
  END; (* func.GermanUpcase *)


    (* ------------------------ Main Program ----------------------- *)


BEGIN (* main *)
  (* Stimmt die Parameteranzahl? *)
  IF (ParamCount >= 2) AND (ParamCount <= 4) THEN BEGIN
    (* Der erste Parameter ist immer der Treibername: *)
    DriverFile := ParamStr(1);

    debug := FALSE; (* Standardmaessig ist der Debug-Modus ausgeschaltet. *)
    KommaPunkt := 0; (* Bedeutet: Kein Patchen des Nummernblocks *)
    ShiftCaps  := 0; (* Bedeutet: Kein Patchen der Locking-Funktion *)

    (* Ermittle die restlichen Parameter: *)
    IF (GermanUpcase(ParamStr(2)) = '-KOMMA') THEN
      KommaPunkt := 44;
    IF (GermanUpcase(ParamStr(2)) = '-PUNKT') THEN
      KommaPunkt := 46;
    IF (GermanUpcase(ParamStr(2)) = '-SHIFTLOCK') THEN
      ShiftCaps := 1;
    IF (GermanUpcase(ParamStr(2)) = '-CAPSLOCK') THEN
      ShiftCaps := 2;
    IF (ParamCount = 3) THEN BEGIN
      (* Den ganzen Sums nochmal: *)
      IF (GermanUpcase(ParamStr(3)) = '-KOMMA') THEN
        KommaPunkt := 44;
      IF (GermanUpcase(ParamStr(3)) = '-PUNKT') THEN
        KommaPunkt := 46;
      IF (GermanUpcase(ParamStr(3)) = '-SHIFTLOCK') THEN
        ShiftCaps := 1;
      IF (GermanUpcase(ParamStr(3)) = '-CAPSLOCK') THEN
        ShiftCaps := 2;
      IF (GermanUpcase(ParamStr(3)) = '-DEBUG') THEN
        debug := TRUE;
    END; (* if ParamCount = 3 *)
    IF (GermanUpcase(ParamStr(4)) = '-CAPSLOCK') THEN
      debug := TRUE;

    (* Einlesen des uebergebenen Files. Es findet keine Fehlerpruefung statt;
       Dies muss vorher extern (im Batchfile) geschehen sein. *)

    ReadFileInBuffer(DriverFile);
    MustWrite := FALSE;

    (* Ausfuehren der Patches: *)
    IF (KommaPunkt <> 0) THEN
      PatchKommaPunkt(KommaPunkt);
    IF (ShiftCaps = 1) THEN
      PatchShiftLock;
    IF (ShiftCaps = 2) THEN
      PatchCapsLock;

    IF MustWrite THEN
      WriteFileFromBuffer(DriverFile)
    ELSE BEGIN
      (* Da kein Schreiben noetig ist, waren entweder die Parameter falsch
         oder die Stellen im Treiber wurden nicht gefunden! *)
      Halt(2);
    END; (* else MustWrite *)
  END ELSE BEGIN
    (* Falsche Anzahl Parameter - tue gar nichts. *)
    Halt(1);
  END; (* else ParamCount = 2 ... *)

END. (* main *)
