PROGRAM Win3PKB;

(*
   Version 1.0 vom 18.08.1999
   Autor: Dirk Schwarzmann
   EMail: Dirk.Schwarzmann@gmx.de
   WWW:   http://www.informatik.uni-oldenburg.de/~rob/

   Dieses Programm ist speziell fuer das Einfuegen zusaetzlicher Tastatur-
   layouts in MS-Windows 3.x gedacht.

   Parameter beim Aufruf:
     Windows-Verzeichnis


   Arbeitsweise:
   Win3PKB liest die Datei <Windows-Verzeichnis>\SYSTEM\SETUP.INF ein und
   sucht nach dem Anfang (gerdll =) der Zeile [*]:

     gerdll = 2:kbdgr.dll , "Deutsch"

   und fuegt die folgenden drei Zeilen hinzu:

     gradll = 2:kbdgr1.dll , "Deutsch (EDV)"
     grbdll = 2:kbdgr10.dll , "Deutsch (Standard, Punkt)"
     grcdll = 2:kbdgr11.dll , "Deutsch (EDV, Punkt)"


   Die neue Datei wird unter <Windows-Verzeichnis>\SYSTEM\SETUP.ROB
   gespeichert.
   War die Datei schon mal gepatcht worden, wird das erkannt, der Schreib-
   vorgang abgebrochen und die angefangene Datei wieder geloescht.
   In einer Batchdatei kann man also anhand der Existenz der Datei SETUP.ROB
   erkennen, ob SETUP.INF gepatcht wurde oder nicht.

   Startet man Windows, findet man die neuen Layouts unter
      Systemsteuerung->Laendereinstellungen->Tastaturlayout,
   wo man nun eines auswaehlen kann. Die entsprechende Datei wird dann vom
   Installationslaufwerk angefordert.
   Nachdem die Datei installiert wurde, steht das neue Layout sofort zur
   Verfuegung.

   [*] Anmerkung:
       Um unabhaengig von verschiedenen Windows-Versionen zu sein, wird der
       letzte Teil der Zeile ignoriert - in der englischen Version etwa steht
       hier naemlich "German", ausserdem koennte statt der 2 auch eine andere
       Zahl stehen (diese gibt an, auf welcher Diskette der Treiber liegt).

*)

USES
  DOS, CRT;

CONST
  NumLines = 3;  (* Anzahl der einzufuegenden Zeilen *)

  (* String, der gefunden werden muss: *)
  FindString = 'gerdll = ';

  (* Zwischen die beiden Stringteile muss die Nummer der Diskette eingefuegt
     werden, auf der die Treiber normalerweise zu finden sind.
     Dies geht aus der Zahl, die in der gelesenen Zeile hinter "FindString"
     steht, hervor. Ist nicht wirklich wichtig, macht aber das Windows-Setup
     konsistent.
   *)
  AddStrPrefix : ARRAY[1..NumLines] OF STRING =
    ('gradll = ',
     'grbdll = ',
     'grcdll = ');
  AddStrSuffix : ARRAY[1..NumLines] OF STRING =
    (':kbdgr1.dll , "Deutsch (EDV)"',
     ':kbdgr10.dll , "Deutsch (Standard, Punkt)"',
     ':kbdgr11.dll , "Deutsch (EDV, Punkt)"');

VAR
  InFile   : TEXT;   (* Handle fuer die Originaldatei *)
  OutFile  : TEXT;   (* Handle fuer die neue Datei *)
  InString : STRING; (* Aktuell eingelesene Zeile der Datei *)
  i        : BYTE;
  WasPatched : BOOLEAN; (* Flag, ob SETUP.INF schon mal gepatcht wurde *)
  DiskNumber : STRING;  (* Enthaelt die Nummer der Diskette, auf der die
                           Treiber laut Setup gespeichert sind. *)

BEGIN
  WasPatched := FALSE;

  (* Oeffne die Originaldatei: *)
  Assign(InFile,ParamStr(1)+'\SYSTEM\SETUP.INF');
  Reset(InFile);
  (* Lege die neue Datei an: *)
  Assign(OutFile,ParamStr(1)+'\SYSTEM\SETUP.ROB');
  ReWrite(OutFile);

  (* Lies die Originaldatei und schreibe die neue: *)
  REPEAT
    ReadLn(InFile, InString);
    (* Kopiere gleich in die neue Datei: *)
    WriteLn(OutFile, InString);

    (* Teste, ob die Datei schon gepatcht war.
       Dazu pruefe, ob die aktuelle Zeile der ersten einzufuegenden
       entspricht. Wenn ja, war die Datei schon gepatcht.
    *)
    IF (AddStrPrefix[1] <> Copy(InString,1,Length(AddStrPrefix[1]))) THEN BEGIN
      (* (Noch) nicht gefunden, also vergleiche die Zeile mit der gesuchten: *)
      IF (Copy(InString,1,Length(FindString)) = FindString) THEN BEGIN
        (* Zeile gefunden. Nun lies die Zahl, die angibt, wo der Treiber
           gespeichert ist. Nimm dann diese Zahl auch fuer die neuen Treiber.
         *)
        DiskNumber := Copy(InString,Length(FindString)+1,1);
        (* Schreibe die einzufuegenden Zeilen: *)
        FOR i := 1 TO NumLines DO
          WriteLn(OutFile, AddStrPrefix[i]+DiskNumber+AddStrSuffix[i]);
      END; (* if (Copy(InString,1,... *)
    END ELSE (* if InString <>... *)
      (* Die erste der einzufuegenden Zeilen wurde gefunden. Also koennen
         wir Schluss machen. Die Datei wird dann geloescht (s.u.) *)
      WasPatched := TRUE;
  UNTIL EoF(InFile) OR WasPatched;
  Close(OutFile);
  Close(InFile);

  (* Die schon angefangene Datei kann geloescht werden, weil sie ja schon
     existiert (von einem frueheren Patch) *)
  IF WasPatched THEN
    Erase(OutFile);
END.