PROGRAM Choice;
(*
   Version 1.0 vom 1.10.1999
   Autor: Dirk Schwarzmann
   EMail: Dirk.Schwarzmann@igd.fhg.de
   WWW:   http://www.informatik.uni-oldenburg.de/~rob/


   Beschreibung:
     Das Programm stellt einen Ersatz fuer das DOS-Programm CHOICE.COM dar,
     d.h. es liest ein einzelnes Zeichen von der Tastatur ein und beendet sich
     gemaess der Eingabe mit einem entsprechenden Errorlevel, der von der
     aufrufenden Batchdatei abgefragt werden kann.
     Als Erweiterung zum originalen CHOICE.COM kann CHOICE.EXE auch zwischen
     Gross- und Kleinbuchstaben unterscheiden, wenn man den entsprechenden
     Parameter angibt. Standardmaessig unterscheidet es nicht.
     Einziger Grund fuer dieses Programm ist, dass es von PatchKeyboard 4.x
     benoetigt wird und unter Windows NT 4.0 nicht mehr vorhanden ist.
     Aus urheberrechtlichen Gruenden ist es auch nicht erlaubt, CHOICE.COM
     von Microsoft in eigenen Programmen mitzuliefern.

   Parameter beim Aufruf:
     /c:<Liste der zu akzeptierenden Zeichen>
    -case (optional; muss immer der letzte Parameter sein)

   Beispiele:
     CHOICE.EXE /c:abc123          (akzeptiert die Zeichen a,A,b,B,c,C,123)
     CHOICE.EXE /c:abc123 -case    (akzeptiert die Zeichen a,b,c,123)
     CHOICE.EXE /c:abcABC12 -case  (akzeptiert die Zeichen a,A,b,B,c,C,1,2)

   Rueckgabewerte (Abfrage per "ERRORLEVEL" in einer Batchdatei):
     Ein Errorlevel von 1 bis zur Anzahl der zu pruefenden Zeichen.
     Errorlevel 0 steht fuer einen falschen Aufruf (falsche Parameteranzahl,
     falscher (nicht unterstuetzter) Parameter).

   Einschraenkungen:
     CHOICE.EXE hat nicht den Funktionsumfang des DOS-Programms CHOICE.COM.
     Es wird lediglich die Liste zu akzeptierender Zeichen sowie die Ausgabe
     des entsprechenden Errorlevels unterstuetzt (weil mehr in PKB nicht
     gebraucht wird).

   ACHTUNG! Die Errorlevel muessen rueckwaerts, d.h. mit der hoechsten Zahl
   beginnend abgefragt werden, sonst wird immer "ok" erkannt!
   (Dies ist ein Problem von DOS und hat nichts mit diesem Programm zu tun)

*)


USES
  DOS, CRT;

VAR
  Parameter   : STRING;  (* der P., der sagt, was getan werden soll *)
  CheckKeys   : STRING;  (* enthaelt die zu pruefenden Zeichen *)
  key         : CHAR;    (* eingelesenes Zeichen von der Tastatur *)
  keystr      : STRING;  (* enthaelt ebenfalls das eingel. Zeichen, fuer
                            Funktion GermanUpcase noetig (Typkonvertierung) *)
  keyPosition : INTEGER; (* Position des gefundenen Zeichens *)


FUNCTION GermanUpcase(WorkStr:STRING):STRING;
(*
   Konvertiert in einem String alle Buchstaben samt Umlauten in
   Grossbuchstaben.
 *)

  VAR
    i : BYTE;

  BEGIN (* func.GermanUpcase *)
    FOR i := 1 To Length(WorkStr) DO BEGIN
      WorkStr[i] := Upcase(WorkStr[i]);
      CASE WorkStr[i] OF
        '' : WorkStr[i] := '';
        '' : WorkStr[i] := '';
        '' : WorkStr[i] := '';
      END; (* case WorkStr[i] *)
    END; (* for i *)
    GermanUpcase := WorkStr;
  END; (* func.GermanUpcase *)

    (* ------------------------ Main Program ----------------------- *)

BEGIN (* main *)
  (* Stimmt die Parameteranzahl? *)
  IF (ParamCount < 1) OR (ParamCount > 2) THEN BEGIN
    (* Falsche Parameterzahl, beende Programm mit Errorlevel 0 *)
    Halt(0);
  END ELSE BEGIN
    (* Ok, wir koennen anfangen.
       Zunaechst checken, ob der Parameter ueberhaupt unterstuetzt wird: *)
    Parameter := Copy(ParamStr(1),1,3);
    Parameter[2] := Upcase(Parameter[2]);
    IF (Parameter = '/C:') THEN BEGIN
      (* Alles klar. Kopiere die zu pruefenden Zeichen in den Arbeitsstring: *)
      CheckKeys := Copy(ParamStr(1),4,255);
      (* Behandlung haengt davon ab, ob der Parameter -case gesetzt wurde: *)
      IF (GermanUpcase(ParamStr(2)) = '-CASE') THEN BEGIN
        (* Beachte Gross-/Kleinschreibung *)
        REPEAT
          key := ReadKey; (* lies ein Zeichen von der Tastatur *)
          (* ist das Zeichen enthalten? Wenn ja, liefert Pos die Position, an
             der das Zeichen gefunden wurde, sonst 0 *)
          keyPosition := Pos(key,CheckKeys);
        UNTIL (keyPosition > 0);
      END ELSE BEGIN
        (* Gross-/Kleinschreibung unwichtig *)
        (* Gross/Klein normalisieren: *)
        CheckKeys := GermanUpcase(CheckKeys);
        REPEAT
          key := ReadKey; (* lies ein Zeichen von der Tastatur *)
          keystr := GermanUpcase(key);
          key := keystr[1];
          keyPosition := Pos(key,CheckKeys);
        UNTIL (keyPosition > 0);
      END; (* if (GermanUpcase(ParamStr(2) = '-CASE') *)
      Halt(keyPosition);
    END ELSE BEGIN
      (* Falscher Parameter. Beenden mit Errorlevel 0 *)
      Halt(0);
    END; (* if (Parameter = '/C:') *)
  END; (* if (ParamCount <> 1) *)
END. (* main *)
