// ------------------------------------------------------------------------ //
//                                                                          //
// File:    pCase.cpp                                                       //
// Author:  Dirk Schwarzmann                                                //
// Contact: Dirk.Schwarzmann@gmx.de                                         //
//                                                                          //
// Version: 1.0                                                             //
// Date:    23th July 2000                                                  //
//                                                                          //
// Purpose:                                                                 //
//          Takes a parameter from the command line and converts it to      //
//          either upper case or lower case.                                //
//          Output is directed to standard out, so you can redirect it to   //
//          to a file or whatever.                                          //
//                                                                          //
//          Case is chosen by a second parameter (-lower or -upper)         //
//          If no case is given, Case by default converts to upper case.    //
//                                                                          //
//          The third parameter, if specified, tell pCase to prepend the    //
//          parameter as variable name to be set. The call and output then  //
//          will look as such:                                              //
//                                                                          //
//          call: pcase Test -lower param                                   //
//                                                                          //
//          output: set param=test                                          //
//                                                                          //
// ------------------------------------------------------------------------ //

#include <iostream.h>
#include <string.h>
#include <ctype.h>


void main (int argc, char* argv[]) {

	// If no parameter is given, do not do anything:
	if (argc > 1) {

		// Process the first parameter according to the optional second one:
		if ((argc > 2) && (strcmp(argv[2], "-lower") == 0)) {

			for (unsigned int i=0; i<strlen(argv[1]); i++) {

				argv[1][i] = tolower(argv[1][i]);
			}

		} else {

			for (unsigned int i=0; i<strlen(argv[1]); i++) {

				argv[1][i] = toupper(argv[1][i]);
			}
		}

		if (argc > 3) {
		
			cout << "set " << argv[3] << "=" << argv[1] << endl;

		} else {

			cout << argv[1] << endl;
		}

	} else { // if (argc > 1)

		// print a help page
		cout << "pCase.exe V1.0, 23th July 2000 by Dirk Schwarzmann\n";
		cout << "Syntax:\n";
		cout << "  pcase Text [-lower [parameter]]\n";
		cout << "\n'Text' is the string to be converted to upper or lower case.\n";
		cout << "By default, pCase converts to upper case. By specifying the keyword '-lower',\n";
		cout << "'Text' becomes 'text'. The optional third keyword forces pCase to produce the\n";
		cout << "following output: 'set parameter=text' which you can direct to a file.\n\n";
	}

}; // main (int, char*)